@@ -5,11 +5,12 @@
 
     Other formatters: NullFormatter, RawTokenFormatter.
 
-    :copyright: 2006 by Georg Brandl, Armin Ronacher.
+    :copyright: 2006-2007 by Georg Brandl, Armin Ronacher.
     :license: BSD, see LICENSE for more details.
 """
 
 from pygments.formatter import Formatter
+from pygments.util import get_choice_opt
 
 
 __all__ = ['NullFormatter', 'RawTokenFormatter']
@@ -19,6 +20,10 @@ class NullFormatter(Formatter):
     """
     Output the text unchanged without any formatting.
     """
+    name = 'Text only'
+    aliases = ['text', 'null']
+    filenames = ['*.txt']
+
     def format(self, tokensource, outfile):
         enc = self.encoding
         for ttype, value in tokensource:
@@ -29,23 +34,29 @@ class NullFormatter(Formatter):
 
 
 class RawTokenFormatter(Formatter):
+    r"""
+    Format tokens as a raw representation for storing token streams.
+
+    The format is ``tokentype<TAB>repr(tokenstring)\n``. The output can later
+    be converted to a token stream with the `RawTokenLexer`, described in the
+    `lexer list <lexers.txt>`_.
+
+    Only one option is accepted:
+
+    `compress`
+        If set to ``'gz'`` or ``'bz2'``, compress the output with the given
+        compression algorithm after encoding (default: ``''``).
     """
-    Output a raw token representation for storing token streams.
-
-    The format is ``tokentype<TAB>repr(tokenstring)``
-
-    Additional options accepted:
-
-    ``compress``
-        If set to "gz" or "bz2", compress the token stream with
-        the given compression algorithm (default: '').
-    """
+    name = 'Raw tokens'
+    aliases = ['raw', 'tokens']
+    filenames = ['*.raw']
 
     unicodeoutput = False
 
     def __init__(self, **options):
         Formatter.__init__(self, **options)
-        self.compress = options.get('compress', '')
+        self.compress = get_choice_opt(options, 'compress',
+                                       ['', 'none', 'gz', 'bz2'], '')
 
     def format(self, tokensource, outfile):
         if self.compress == 'gz':
@@ -68,13 +79,5 @@ class RawTokenFormatter(Formatter):
         lasttype = None
         lastval = u''
         for ttype, value in tokensource:
-            value = repr(value)
-            if ttype is lasttype:
-                lastval += value
-            else:
-                if lasttype:
-                    write("%s\t%s\n" % (lasttype, lastval))
-                lastval = value
-                lasttype = ttype
-        write("%s\t%s\n" % (lasttype, lastval))
+            write("%s\t%r\n" % (ttype, value))
         flush()
