@@ -5,8 +5,8 @@
 
     Lexers for compiled languages.
 
-    :copyright: 2006-2007 by Georg Brandl, Armin Ronacher, Christoph Hack,
-                Whitney Young, Kirk McDonald.
+    :copyright: 2006-2008 by Georg Brandl, Armin Ronacher, Christoph Hack,
+                Whitney Young, Kirk McDonald, Stou Sandalski.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -24,9 +24,11 @@ from pygments.token import \
      Text, Comment, Operator, Keyword, Name, String, Number, Punctuation, \
      Error
 
+# backwards compatibility
+from pygments.lexers.functional import OcamlLexer
 
 __all__ = ['CLexer', 'CppLexer', 'DLexer', 'DelphiLexer', 'JavaLexer',
-           'DylanLexer', 'OcamlLexer', 'ObjectiveCLexer']
+           'DylanLexer', 'OcamlLexer', 'ObjectiveCLexer', 'FortranLexer']
 
 
 class CLexer(RegexLexer):
@@ -61,6 +63,7 @@ class CLexer(RegexLexer):
             (r'\d+[Ll]?', Number.Integer),
             (r'[~!%^&*+=|?:<>/-]', Operator),
             (r'[()\[\],.]', Punctuation),
+            (r'\b(case)(.+?)(:)', bygroups(Keyword, using(this), Text)),
             (r'(auto|break|case|const|continue|default|do|else|enum|extern|'
              r'for|goto|if|register|restricted|return|sizeof|static|struct|'
              r'switch|typedef|union|volatile|virtual|while)\b', Keyword),
@@ -132,7 +135,7 @@ class CppLexer(RegexLexer):
     """
     name = 'C++'
     aliases = ['cpp', 'c++']
-    filenames = ['*.cpp', '*.hpp', '*.c++', '*.h++']
+    filenames = ['*.cpp', '*.hpp', '*.c++', '*.h++', '*.cc', '*.hh', '*.cxx', '*.hxx']
     mimetypes = ['text/x-c++hdr', 'text/x-c++src']
 
     tokens = {
@@ -220,7 +223,7 @@ class DLexer(RegexLexer):
             # Comments
             (r'//(.*?)\n', Comment),
             (r'/(\\\n)?[*](.|\n)*?[*](\\\n)?/', Comment),
-            (r'/\+', Comment, 'nestedcomment'),
+            (r'/\+', Comment, 'nested_comment'),
             # Keywords
             (r'(abstract|alias|align|asm|assert|auto|body|break|case|cast'
              r'|catch|class|const|continue|debug|default|delegate|delete'
@@ -230,7 +233,7 @@ class DLexer(RegexLexer):
              r'|package|pragma|private|protected|public|ref|return|scope'
              r'|static|struct|super|switch|synchronized|template|this|throw'
              r'|try|typedef|typeid|typeof|union|unittest|version|volatile'
-             r'|while|with)\b', Keyword
+             r'|while|with|__traits)\b', Keyword
             ),
             (r'(bool|cdouble|cent|cfloat|char|creal|dchar|double|float|idouble'
              r'|ifloat|int|ireal|long|real|short|ubyte|ucent|uint|ulong|ushort'
@@ -238,6 +241,7 @@ class DLexer(RegexLexer):
             ),
             (r'(false|true|null)\b', Keyword.Constant),
             (r'macro\b', Keyword.Reserved),
+            (r'(string|wstring|dstring)\b', Name.Builtin),
             # FloatLiteral
             # -- HexFloat
             (r'0[xX]([0-9a-fA-F_]*\.[0-9a-fA-F_]+|[0-9a-fA-F_]+)'
@@ -262,32 +266,91 @@ class DLexer(RegexLexer):
             ),
             # StringLiteral
             # -- WysiwygString
-            (r'(?s)r"[^"]*"[cwd]?', String),
+            (r'r"[^"]*"[cwd]?', String),
             # -- AlternateWysiwygString
-            (r'(?s)`[^`]*`[cwd]?', String),
+            (r'`[^`]*`[cwd]?', String),
             # -- DoubleQuotedString
-            (r'(?s)"(\\"|[^"])*"[cwd]?', String),
+            (r'"(\\\\|\\"|[^"])*"[cwd]?', String),
             # -- EscapeSequence
             (r"""\\(['"?\\abfnrtv]|x[0-9a-fA-F]{2}|[0-7]{1,3}"""
              r"""|u[0-9a-fA-F]{4}|U[0-9a-fA-F]{8}|&\w+;)""",
              String
             ),
             # -- HexString
-            (r'(?s)x"[0-9a-fA-F_\s]*"[cwd]?', String),
+            (r'x"[0-9a-fA-F_\s]*"[cwd]?', String),
+            # -- DelimitedString
+            (r'q"\[', String, 'delimited_bracket'),
+            (r'q"\(', String, 'delimited_parenthesis'),
+            (r'q"<', String, 'delimited_angle'),
+            (r'q"{', String, 'delimited_curly'),
+            (r'q"([a-zA-Z_]\w*)\n.*?\n\1"', String),
+            (r'q"(.).*?\1"', String),
+            # -- TokenString
+            (r'q{', String, 'token_string'),
             # Tokens
             (r'(~=|\^=|%=|\*=|==|!>=|!<=|!<>=|!<>|!<|!>|!=|>>>=|>>>|>>=|>>|>='
              r'|<>=|<>|<<=|<<|<=|\+\+|\+=|--|-=|\|\||\|=|&&|&=|\.\.\.|\.\.|/=)'
-             r'|[/.&|\-+<>!()\[\]{}?,;:$=*%^~]', Text #Punctuation
+             r'|[/.&|\-+<>!()\[\]{}?,;:$=*%^~]', Punctuation
             ),
             # Identifier
             (r'[a-zA-Z_]\w*', Name),
         ],
-        'nestedcomment': [
+        'nested_comment': [
             (r'[^+/]+', Comment),
             (r'/\+', Comment, '#push'),
             (r'\+/', Comment, '#pop'),
             (r'[+/]', Comment),
         ],
+        'token_string': [
+            (r'{', Punctuation, 'token_string_nest'),
+            (r'}', String, '#pop'),
+            include('root'),
+        ],
+        'token_string_nest': [
+            (r'{', Punctuation, '#push'),
+            (r'}', Punctuation, '#pop'),
+            include('root'),
+        ],
+        'delimited_bracket': [
+            (r'[^\[\]]+', String),
+            (r'\[', String, 'delimited_inside_bracket'),
+            (r'\]"', String, '#pop'),
+        ],
+        'delimited_inside_bracket': [
+            (r'[^\[\]]+', String),
+            (r'\[', String, '#push'),
+            (r'\]', String, '#pop'),
+        ],
+        'delimited_parenthesis': [
+            (r'[^\(\)]+', String),
+            (r'\(', String, 'delimited_inside_parenthesis'),
+            (r'\)"', String, '#pop'),
+        ],
+        'delimited_inside_parenthesis': [
+            (r'[^\(\)]+', String),
+            (r'\(', String, '#push'),
+            (r'\)', String, '#pop'),
+        ],
+        'delimited_angle': [
+            (r'[^<>]+', String),
+            (r'<', String, 'delimited_inside_angle'),
+            (r'>"', String, '#pop'),
+        ],
+        'delimited_inside_angle': [
+            (r'[^<>]+', String),
+            (r'<', String, '#push'),
+            (r'>', String, '#pop'),
+        ],
+        'delimited_curly': [
+            (r'[^{}]+', String),
+            (r'{', String, 'delimited_inside_curly'),
+            (r'}"', String, '#pop'),
+        ],
+        'delimited_inside_curly': [
+            (r'[^{}]+', String),
+            (r'{', String, '#push'),
+            (r'}', String, '#pop'),
+        ],
     }
 
 
@@ -793,13 +856,10 @@ class JavaLexer(RegexLexer):
     tokens = {
         'root': [
             # method names
-            (r'^(\s*(?:[a-zA-Z_][a-zA-Z0-9_\.]*\s+)+?)'  # return arguments
-             r'([a-zA-Z_][a-zA-Z0-9_]*)'                 # method name
-             r'(\s*\([^;]*?\))'                          # signature
-             r'(?=' + _ws +                              # exception declaration
-             r'(?:throws\s+(?:[a-zA-Z_][a-zA-Z0-9_]*,?\s*)+)?' +
-             _ws + r'\{)',
-             bygroups(using(this), Name.Function, using(this))),
+            (r'^(\s*(?:[a-zA-Z_][a-zA-Z0-9_\.\[\]]*\s+)+?)' # return arguments
+             r'([a-zA-Z_][a-zA-Z0-9_]*)'                    # method name
+             r'(\s*)(\()',                                  # signature start
+             bygroups(using(this), Name.Function, Text, Operator)),
             (r'[^\S\n]+', Text),
             (r'//.*?\n', Comment),
             (r'/\*.*?\*/', Comment),
@@ -807,13 +867,13 @@ class JavaLexer(RegexLexer):
             (r'(abstract|assert|break|case|catch|'
              r'const|continue|default|do|else|enum|extends|final|'
              r'finally|for|if|goto|implements|instanceof|'
-             r'interface|native|new|package|private|protected|public|'
+             r'native|new|package|private|protected|public|'
              r'return|static|strictfp|super|switch|synchronized|this|'
              r'throw|throws|transient|try|volatile|while)\b', Keyword),
             (r'(boolean|byte|char|double|float|int|long|short|void)\b',
              Keyword.Type),
             (r'(true|false|null)\b', Keyword.Constant),
-            (r'(class)(\s+)', bygroups(Keyword, Text), 'class'),
+            (r'(class|interface)(\s+)', bygroups(Keyword, Text), 'class'),
             (r'(import)(\s+)', bygroups(Keyword, Text), 'import'),
             (r'"(\\\\|\\"|[^"])*"', String),
             (r"'\\.'|'[^\\]'|'\\u[0-9a-f]{4}'", String.Char),
@@ -842,7 +902,7 @@ class DylanLexer(RegexLexer):
     *New in Pygments 0.7.*
     """
 
-    name = 'DylanLexer'
+    name = 'Dylan'
     aliases = ['dylan']
     filenames = ['*.dylan']
     mimetypes = ['text/x-dylan']
@@ -880,86 +940,6 @@ class DylanLexer(RegexLexer):
     }
 
 
-class OcamlLexer(RegexLexer):
-    """
-    For the OCaml language.
-
-    *New in Pygments 0.7.*
-    """
-
-    name = 'OCaml'
-    aliases = ['ocaml']
-    filenames = ['*.ml', '*.mli']
-    mimetypes = ['text/x-ocaml']
-
-    keywords = [
-      'and', 'as', 'assert', 'asr', 'begin', 'class',
-      'constraint', 'do', 'done', 'downto', 'else', 'end',
-      'exception', 'external', 'false', 'for', 'fun', 'function',
-      'functor', 'if', 'in', 'include', 'inherit', 'initializer',
-      'land', 'lazy', 'let', 'lor', 'lsl', 'lsr',
-      'lxor', 'match', 'method', 'mod', 'module', 'mutable',
-      'new', 'object', 'of', 'open', 'or', 'private',
-      'rec', 'sig', 'struct', 'then', 'to', 'true',
-      'try', 'type', 'val', 'virtual', 'when', 'while', 'with'
-    ]
-    keyopts = [
-      '!=','#','&','&&','\(','\)','\*','\+',',','-',
-      '-\.','->','\.','\.\.',':','::',':=',':>',';',';;','<',
-      '<-','=','>','>]','>}','\?','\?\?','\[','\[<','\[>','\[\|',
-      ']','_','`','{','{<','\|','\|]','}','~'
-    ]
-
-    operators = r'[!$%&*+\./:<=>?@^|~-]'
-    prefix_syms = r'[!?~]'
-    infix_syms = r'[=<>@^|&+\*/$%-]'
-
-    tokens = {
-        'escape-sequence': [
-            (r'\\[\"\'ntbr]', String.Escape),
-            (r'\\[0-9]{3}', String.Escape),
-            (r'\\x[0-9a-fA-F]{2}', String.Escape),
-        ],
-        'root': [
-            (r'\s+', Text),
-            (r'\(\*', Comment, 'comment'),
-            (r'\b(%s)\b' % '|'.join(keywords), Keyword.Reserved),
-            (r'(%s)' % '|'.join(keyopts), Keyword),
-            (r'false|true|\(\)|\[\]', Name.Constant),
-            (r'(%s|%s)?%s' % (infix_syms, prefix_syms, operators), Operator),
-
-            (r"[^\W\d][\w']*", Name),
-
-            (r'\d[\d_]*', Number.Integer),
-            (r'0[xX][\da-fA-F][\da-fA-F_]*', Number.Hex),
-            (r'0[oO][0-7][0-7_]*', Number.Oct),
-            (r'0[bB][01][01_]*', Number),
-            (r'-?\d[\d_]*(.[\d_]*)?([eE][+\-]?\d[\d_]*)', Number.Float),
-
-            (r"'(?:(\\[\\\"'ntbr ])|(\\[0-9]{3})|(\\x[0-9a-fA-F]{2}))'",
-             String.Char),
-            (r"'.'", String.Char),
-            (r"'", Keyword), # a stray quote is another syntax element
-
-            (r'"', String.Double, 'string'),
-
-            (r'[~?][a-z][\w\']*:', Name.Variable),
-        ],
-        'comment': [
-            (r'[^(*)]', Comment),
-            (r'\(\*', Comment, '#push'),
-            (r'\*\)', Comment, '#pop'),
-            (r'[(*)]', Comment),
-        ],
-        'string': [
-            (r'[^\\"]', String.Double),
-            include('escape-sequence'),
-            (r'\\\n', String.Double),
-            (r'"', String.Double, '#pop'),
-        ]
-    }
-
-
 class ObjectiveCLexer(RegexLexer):
     """
     For Objective-C source code with preprocessor directives.
@@ -1078,3 +1058,104 @@ class ObjectiveCLexer(RegexLexer):
             (r'.*?\n', Comment),
         ]
     }
+
+
+class FortranLexer(RegexLexer):
+    '''
+    Lexer for FORTRAN 90 code.
+
+    *New in Pygments 0.10.*
+    '''
+    name = 'Fortran'
+    aliases = ['fortran']
+    filenames = ['*.f', '*.f90']
+    mimetypes = ['text/x-fortran']
+    flags = re.IGNORECASE
+
+    # Data Types: INTEGER, REAL, COMPLEX, LOGICAL, CHARACTER and DOUBLE PRECISION
+    # Operators: **, *, +, -, /, <, >, <=, >=, ==, /=
+    # Logical (?): NOT, AND, OR, EQV, NEQV
+
+    # Builtins:
+    # http://gcc.gnu.org/onlinedocs/gcc-3.4.6/g77/Table-of-Intrinsic-Functions.html
+
+    tokens = {
+        'root': [
+            (r'!.*\n', Comment),
+            include('strings'),
+            include('core'),
+            (r'[a-z][a-z0-9_]*', Name.Variable),
+            include('nums'),
+            (r'[\s]+', Text),
+        ],
+        'core': [
+            # Statements
+            (r'\b(ACCEPT|ALLOCATABLE|ALLOCATE|ARRAY|ASSIGN|BACKSPACE|BLOCK DATA|'
+             r'BYTE|CALL|CASE|CLOSE|COMMON|CONTAINS|CONTINUE|CYCLE|DATA|'
+             r'DEALLOCATE|DECODE|DIMENSION|DO|ENCODE|END FILE|ENDIF|END|ENTRY|'
+             r'EQUIVALENCE|EXIT|EXTERNAL|EXTRINSIC|FORALL|FORMAT|FUNCTION|GOTO|'
+             r'IF|IMPLICIT|INCLUDE|INQUIRE|INTENT|INTERFACE|INTRINSIC|MODULE|'
+             r'NAMELIST|NULLIFY|NONE|OPEN|OPTIONAL|OPTIONS|PARAMETER|PAUSE|'
+             r'POINTER|PRINT|PRIVATE|PROGRAM|PUBLIC|PURE|READ|RECURSIVE|RETURN|'
+             r'REWIND|SAVE|SELECT|SEQUENCE|STOP|SUBROUTINE|TARGET|TYPE|USE|'
+             r'VOLATILE|WHERE|WRITE|WHILE|THEN|ELSE|ENDIF)\s*\b',
+             Keyword),
+
+            # Data Types
+            (r'\b(CHARACTER|COMPLEX|DOUBLE PRECISION|DOUBLE COMPLEX|INTEGER|'
+             r'LOGICAL|REAL)\s*\b',
+             Keyword.Type),
+
+            # Operators
+            (r'(\*\*|\*|\+|-|\/|<|>|<=|>=|==|\/=|=)', Operator),
+
+            (r'(::)', Keyword.Declaration),
+
+            (r'[(),:&%]', Punctuation),
+
+            # Intrinsics
+            (r'\b(Abort|Abs|Access|AChar|ACos|AdjustL|AdjustR|AImag|AInt|Alarm|'
+             r'All|Allocated|ALog|AMax|AMin|AMod|And|ANInt|Any|'
+             r'ASin|Associated|ATan|BesJ|BesJN|BesY|BesYN|'
+             r'Bit_Size|BTest|CAbs|CCos|Ceiling|CExp|Char|ChDir|ChMod|CLog|'
+             r'Cmplx|Complex|Conjg|Cos|CosH|Count|CPU_Time|CShift|CSin|CSqRt|'
+             r'CTime|DAbs|DACos|DASin|DATan|Date_and_Time|DbesJ|'
+             r'DbesJ|DbesJN|DbesY|DbesY|DbesYN|Dble|DCos|DCosH|DDiM|DErF|DErFC|'
+             r'DExp|Digits|DiM|DInt|DLog|DLog|DMax|DMin|DMod|DNInt|Dot_Product|'
+             r'DProd|DSign|DSinH|DSin|DSqRt|DTanH|DTan|DTime|EOShift|Epsilon|'
+             r'ErF|ErFC|ETime|Exit|Exp|Exponent|FDate|FGet|FGetC|Float|'
+             r'Floor|Flush|FNum|FPutC|FPut|Fraction|FSeek|FStat|FTell|'
+             r'GError|GetArg|GetCWD|GetEnv|GetGId|GetLog|GetPId|GetUId|'
+             r'GMTime|HostNm|Huge|IAbs|IAChar|IAnd|IArgC|IBClr|IBits|'
+             r'IBSet|IChar|IDate|IDiM|IDInt|IDNInt|IEOr|IErrNo|IFix|Imag|'
+             r'ImagPart|Index|Int|IOr|IRand|IsaTty|IShft|IShftC|ISign|'
+             r'ITime|Kill|Kind|LBound|Len|Len_Trim|LGe|LGt|Link|LLe|LLt|LnBlnk|'
+             r'Loc|Log|Log|Logical|Long|LShift|LStat|LTime|MatMul|Max|'
+             r'MaxExponent|MaxLoc|MaxVal|MClock|Merge|Min|MinExponent|MinLoc|'
+             r'MinVal|Mod|Modulo|MvBits|Nearest|NInt|Not|Or|Pack|PError|'
+             r'Precision|Present|Product|Radix|Rand|Random_Number|Random_Seed|'
+             r'Range|Real|RealPart|Rename|Repeat|Reshape|RRSpacing|RShift|Scale|'
+             r'Scan|Second|Selected_Int_Kind|Selected_Real_Kind|Set_Exponent|'
+             r'Shape|Short|Sign|Signal|SinH|Sin|Sleep|Sngl|Spacing|Spread|SqRt|'
+             r'SRand|Stat|Sum|SymLnk|System|System_Clock|Tan|TanH|Time|'
+             r'Tiny|Transfer|Transpose|Trim|TtyNam|UBound|UMask|Unlink|Unpack|'
+             r'Verify|XOr|ZAbs|ZCos|ZExp|ZLog|ZSin|ZSqRt)\s*\b',
+             Name.Builtin),
+
+            # Booleans
+            (r'\.(true|false)\.', Name.Builtin),
+            # Comparing Operators
+            (r'\.(eq|ne|lt|le|gt|ge|not|and|or|eqv|neqv)\.', Operator.Word),
+        ],
+
+        'strings': [
+            (r'"(\\\\|\\[0-7]+|\\.|[^"])*"', String.Double),
+            (r"'(\\\\|\\[0-7]+|\\.|[^'])*'", String.Single),
+        ],
+
+        'nums': [
+            (r'\d+(?![.Ee])', Number.Integer),
+            (r'[+-]?\d*\.\d+([eE][-+]?\d+)?', Number.Float),
+            (r'[+-]?\d+\.\d*([eE][-+]?\d+)?', Number.Float),
+        ],
+    }
