@@ -3,10 +3,10 @@
     pygments.lexers.web
     ~~~~~~~~~~~~~~~~~~~
 
-    Lexers for web-related languages: JavaScript, CSS, HTML, XML, PHP.
+    Lexers for web-related languages and markup.
 
-    :copyright: 2006 by Georg Brandl, Armin Ronacher,
-                Tim Hatch <tim@timhatch.com>.
+    :copyright: 2006-2008 by Georg Brandl, Armin Ronacher,
+                Tim Hatch <tim@timhatch.com>, Stou Sandalski.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -24,10 +24,14 @@ from pygments.util import get_bool_opt, get_list_opt, looks_like_xml, \
 
 
 __all__ = ['HtmlLexer', 'XmlLexer', 'JavascriptLexer', 'CssLexer',
-           'PhpLexer']
+           'PhpLexer', 'ActionScriptLexer', 'XsltLexer']
 
 
 class JavascriptLexer(RegexLexer):
+    """
+    For JavaScript source code.
+    """
+
     name = 'JavaScript'
     aliases = ['js', 'javascript']
     filenames = ['*.js']
@@ -37,10 +41,11 @@ class JavascriptLexer(RegexLexer):
     tokens = {
         'root': [
             (r'\s+', Text),
+            (r'<!--', Comment),
             (r'//.*?\n', Comment),
             (r'/\*.*?\*/', Comment),
             (r'/(\\\\|\\/|[^/\n])*/[gim]*', String.Regex),
-            (r'[~\^\*!%&<>\|+=:;,/?-\\]+', Operator),
+            (r'[~\^\*!%&<>\|+=:;,/?\\-]+', Operator),
             (r'[{}\[\]();.]+', Punctuation),
             (r'(for|in|while|do|break|return|continue|if|else|throw|try|'
              r'catch|var|with|const|label|function|new|typeof|'
@@ -52,7 +57,92 @@ class JavascriptLexer(RegexLexer):
              r'Error|eval|isFinite|isNaN|parseFloat|parseInt|document|this|'
              r'window)\b', Name.Builtin),
             (r'[$a-zA-Z_][a-zA-Z0-9_]*', Name.Other),
-            (r'[0-9]+', Number),
+            (r'[0-9][0-9]*\.[0-9]+([eE][0-9]+)?[fd]?', Number.Float),
+            (r'0x[0-9a-fA-F]+', Number.Hex),
+            (r'[0-9]+', Number.Integer),
+            (r'"(\\\\|\\"|[^"])*"', String.Double),
+            (r"'(\\\\|\\'|[^'])*'", String.Single),
+        ]
+    }
+
+
+class ActionScriptLexer(RegexLexer):
+    """
+    For ActionScript source code.
+
+    *New in Pygments 0.9.*
+    """
+
+    name = 'ActionScript'
+    aliases = ['as', 'actionscript']
+    filenames = ['*.as']
+    mimetypes = ['application/x-actionscript', 'text/x-actionscript',
+                 'text/actionscript']
+
+    flags = re.DOTALL
+    tokens = {
+        'root': [
+            (r'\s+', Text),
+            (r'//.*?\n', Comment),
+            (r'/\*.*?\*/', Comment),
+            (r'/(\\\\|\\/|[^/\n])*/[gim]*', String.Regex),
+            (r'[~\^\*!%&<>\|+=:;,/?\\-]+', Operator),
+            (r'[{}\[\]();.]+', Punctuation),
+            (r'(case|default|for|each|in|while|do|break|return|continue|if|else|'
+             r'throw|try|catch|var|with|new|typeof|arguments|instanceof|this|'
+             r'switch)\b', Keyword),
+            (r'(class|public|final|internal|native|override|private|protected|'
+             r'static|import|extends|implements|interface|intrinsic|return|super|'
+             r'dynamic|function|const|get|namespace|package|set)\b', Keyword.Declaration),
+            (r'(true|false|null|NaN|Infinity|-Infinity|undefined|Void)\b',
+             Keyword.Constant),
+            (r'(Accessibility|AccessibilityProperties|ActionScriptVersion|'
+             r'ActivityEvent|AntiAliasType|ApplicationDomain|AsBroadcaster|Array|'
+             r'AsyncErrorEvent|AVM1Movie|BevelFilter|Bitmap|BitmapData|'
+             r'BitmapDataChannel|BitmapFilter|BitmapFilterQuality|BitmapFilterType|'
+             r'BlendMode|BlurFilter|Boolean|ByteArray|Camera|Capabilities|CapsStyle|'
+             r'Class|Color|ColorMatrixFilter|ColorTransform|ContextMenu|'
+             r'ContextMenuBuiltInItems|ContextMenuEvent|ContextMenuItem|'
+             r'ConvultionFilter|CSMSettings|DataEvent|Date|DefinitionError|'
+             r'DeleteObjectSample|Dictionary|DisplacmentMapFilter|DisplayObject|'
+             r'DisplacmentMapFilterMode|DisplayObjectContainer|DropShadowFilter|'
+             r'Endian|EOFError|Error|ErrorEvent|EvalError|Event|EventDispatcher|'
+             r'EventPhase|ExternalInterface|FileFilter|FileReference|'
+             r'FileReferenceList|FocusDirection|FocusEvent|Font|FontStyle|FontType|'
+             r'FrameLabel|FullScreenEvent|Function|GlowFilter|GradientBevelFilter|'
+             r'GradientGlowFilter|GradientType|Graphics|GridFitType|HTTPStatusEvent|'
+             r'IBitmapDrawable|ID3Info|IDataInput|IDataOutput|IDynamicPropertyOutput'
+             r'IDynamicPropertyWriter|IEventDispatcher|IExternalizable|'
+             r'IllegalOperationError|IME|IMEConversionMode|IMEEvent|int|'
+             r'InteractiveObject|InterpolationMethod|InvalidSWFError|InvokeEvent|'
+             r'IOError|IOErrorEvent|JointStyle|Key|Keyboard|KeyboardEvent|KeyLocation|'
+             r'LineScaleMode|Loader|LoaderContext|LoaderInfo|LoadVars|LocalConnection|'
+             r'Locale|Math|Matrix|MemoryError|Microphone|MorphShape|Mouse|MouseEvent|'
+             r'MovieClip|MovieClipLoader|Namespace|NetConnection|NetStatusEvent|'
+             r'NetStream|NewObjectSample|Number|Object|ObjectEncoding|PixelSnapping|'
+             r'Point|PrintJob|PrintJobOptions|PrintJobOrientation|ProgressEvent|Proxy|'
+             r'QName|RangeError|Rectangle|ReferenceError|RegExp|Responder|Sample|Scene|'
+             r'ScriptTimeoutError|Security|SecurityDomain|SecurityError|'
+             r'SecurityErrorEvent|SecurityPanel|Selection|Shape|SharedObject|'
+             r'SharedObjectFlushStatus|SimpleButton|Socket|Sound|SoundChannel|'
+             r'SoundLoaderContext|SoundMixer|SoundTransform|SpreadMethod|Sprite|'
+             r'StackFrame|StackOverflowError|Stage|StageAlign|StageDisplayState|'
+             r'StageQuality|StageScaleMode|StaticText|StatusEvent|String|StyleSheet|'
+             r'SWFVersion|SyncEvent|SyntaxError|System|TextColorType|TextField|'
+             r'TextFieldAutoSize|TextFieldType|TextFormat|TextFormatAlign|'
+             r'TextLineMetrics|TextRenderer|TextSnapshot|Timer|TimerEvent|Transform|'
+             r'TypeError|uint|URIError|URLLoader|URLLoaderDataFormat|URLRequest|'
+             r'URLRequestHeader|URLRequestMethod|URLStream|URLVariabeles|VerifyError|'
+             r'Video|XML|XMLDocument|XMLList|XMLNode|XMLNodeType|XMLSocket|XMLUI)\b',
+             Name.Builtin),
+            (r'(decodeURI|decodeURIComponent|encodeURI|escape|eval|isFinite|isNaN|'
+             r'isXMLName|clearInterval|fscommand|getTimer|getURL|getVersion|'
+             r'isFinite|parseFloat|parseInt|setInterval|trace|updateAfterEvent|'
+             r'unescape)\b',Name.Function),
+            (r'[$a-zA-Z_][a-zA-Z0-9_]*', Name.Other),
+            (r'[0-9][0-9]*\.[0-9]+([eE][0-9]+)?[fd]?', Number.Float),
+            (r'0x[0-9a-f]+', Number.Hex),
+            (r'[0-9]+', Number.Integer),
             (r'"(\\\\|\\"|[^"])*"', String.Double),
             (r"'(\\\\|\\'|[^'])*'", String.Single),
         ]
@@ -60,6 +150,10 @@ class JavascriptLexer(RegexLexer):
 
 
 class CssLexer(RegexLexer):
+    """
+    For CSS (Cascading Style Sheets).
+    """
+
     name = 'CSS'
     aliases = ['css']
     filenames = ['*.css']
@@ -67,8 +161,6 @@ class CssLexer(RegexLexer):
 
     tokens = {
         'root': [
-            (r'(@media)(\s+)(\w+)(\s*)({)', bygroups(Keyword, Text, String,
-             Text, Punctuation), 'media'),
             include('basics'),
         ],
         'basics': [
@@ -78,14 +170,20 @@ class CssLexer(RegexLexer):
             (r'\:[a-zA-Z0-9_-]+', Name.Decorator),
             (r'\.[a-zA-Z0-9_-]+', Name.Class),
             (r'\#[a-zA-Z0-9_-]+', Name.Function),
+            (r'@[a-zA-Z0-9_-]+', Keyword, 'atrule'),
             (r'[a-zA-Z0-9_-]+', Name.Tag),
             (r'[~\^\*!%&\[\]\(\)<>\|+=@:;,./?-]', Operator),
             (r'"(\\\\|\\"|[^"])*"', String.Double),
             (r"'(\\\\|\\'|[^'])*'", String.Single)
         ],
-        'media': [
+        'atrule': [
+            (r'{', Punctuation, 'atcontent'),
+            (r';', Punctuation, '#pop'),
             include('basics'),
-            (r'}', Punctuation, '#pop')
+        ],
+        'atcontent': [
+            include('basics'),
+            (r'}', Punctuation, '#pop:2'),
         ],
         'content': [
             (r'\s+', Text),
@@ -189,16 +287,21 @@ class CssLexer(RegexLexer):
 
 
 class HtmlLexer(RegexLexer):
+    """
+    For HTML 4 and XHTML 1 markup. Nested JavaScript and CSS is highlighted
+    by the appropriate lexer.
+    """
+
     name = 'HTML'
     aliases = ['html']
-    filenames = ['*.html', '*.htm', '*.xhtml']
+    filenames = ['*.html', '*.htm', '*.xhtml', '*.xslt']
     mimetypes = ['text/html', 'application/xhtml+xml']
 
     flags = re.IGNORECASE | re.DOTALL
     tokens = {
         'root': [
             ('[^<&]+', Text),
-            ('&.*?;', Name.Entity),
+            (r'&\S*?;', Name.Entity),
             (r'\<\!\[CDATA\[.*?\]\]\>', Comment.Preproc),
             ('<!--', Comment, 'comment'),
             (r'<\?.*?\?>', Comment.Preproc),
@@ -216,6 +319,7 @@ class HtmlLexer(RegexLexer):
         'tag': [
             (r'\s+', Text),
             (r'[a-zA-Z0-9_:-]+\s*=', Name.Attribute, 'attr'),
+            (r'[a-zA-Z0-9_:-]+', Name.Attribute),
             (r'/?\s*>', Name.Tag, '#pop'),
         ],
         'script-content': [
@@ -239,9 +343,42 @@ class HtmlLexer(RegexLexer):
 
 
 class PhpLexer(RegexLexer):
+    """
+    For `PHP <http://www.php.net/>`_ source code.
+    For PHP embedded in HTML, use the `HtmlPhpLexer`.
+
+    Additional options accepted:
+
+    `startinline`
+        If given and ``True`` the lexer starts highlighting with
+        php code (i.e.: no starting ``<?php`` required).  The default
+        is ``False``.
+    `funcnamehighlighting`
+        If given and ``True``, highlight builtin function names
+        (default: ``True``).
+    `disabledmodules`
+        If given, must be a list of module names whose function names
+        should not be highlighted. By default all modules are highlighted
+        except the special ``'unknown'`` module that includes functions
+        that are known to php but are undocumented.
+
+        To get a list of allowed modules have a look into the
+        `_phpbuiltins` module:
+
+        .. sourcecode:: pycon
+
+            >>> from pygments.lexers._phpbuiltins import MODULES
+            >>> MODULES.keys()
+            ['PHP Options/Info', 'Zip', 'dba', ...]
+
+        In fact the names of those modules match the module names from
+        the php documentation.
+    """
+
     name = 'PHP'
     aliases = ['php', 'php3', 'php4', 'php5']
     filenames = ['*.php', '*.php[345]']
+    mimetypes = ['text/x-php']
 
     flags = re.IGNORECASE | re.DOTALL | re.MULTILINE
     tokens = {
@@ -256,6 +393,7 @@ class PhpLexer(RegexLexer):
             (r'\s+', Text),
             (r'#.*?\n', Comment),
             (r'//.*?\n', Comment),
+            (r'/\*\*.*?\*/', String.Doc),
             (r'/\*.*?\*/', Comment),
             (r'(->|::)(\s*)([a-zA-Z_][a-zA-Z0-9_]*)',
              bygroups(Operator, Text, Name.Attribute)),
@@ -264,6 +402,8 @@ class PhpLexer(RegexLexer):
             (r'(class)(\s+)', bygroups(Keyword, Text), 'classname'),
             (r'(function)(\s+)(&?)(\s*)',
               bygroups(Keyword, Text, Operator, Text), 'functionname'),
+            (r'(const)(\s+)([a-zA-Z_][a-zA-Z0-9_]*)',
+              bygroups(Keyword, Text, Name.Constant)),
             (r'(and|E_PARSE|old_function|E_ERROR|or|as|E_WARNING|parent|'
              r'eval|PHP_OS|break|exit|case|extends|PHP_VERSION|cfunction|'
              r'FALSE|print|for|require|continue|foreach|require_once|'
@@ -304,7 +444,7 @@ class PhpLexer(RegexLexer):
             (r'(\$\{)(\S+)(\})',
              bygroups(String.Interpol, Name.Variable, String.Interpol)),
             (r'[${\\]+', String.Double)
-        ]
+        ],
     }
 
     def __init__(self, **options):
@@ -335,7 +475,7 @@ class PhpLexer(RegexLexer):
             RegexLexer.get_tokens_unprocessed(self, text, stack):
             if token is Name.Other:
                 if value in self._functions:
-                    yield index, Name.Function, value
+                    yield index, Name.Builtin, value
                     continue
             yield index, token, value
 
@@ -348,11 +488,15 @@ class PhpLexer(RegexLexer):
 
 
 class XmlLexer(RegexLexer):
+    """
+    Generic lexer for XML (eXtensible Markup Language).
+    """
+
     flags = re.MULTILINE | re.DOTALL
 
     name = 'XML'
     aliases = ['xml']
-    filenames = ['*.xml', '*.xsl', '*.rss']
+    filenames = ['*.xml', '*.xsl', '*.rss', '*.xslt', '*.xsd', '*.wsdl']
     mimetypes = ['text/xml', 'application/xml', 'image/svg+xml',
                  'application/rss+xml', 'application/atom+xml',
                  'application/xsl+xml', 'application/xslt+xml']
@@ -360,13 +504,13 @@ class XmlLexer(RegexLexer):
     tokens = {
         'root': [
             ('[^<&]+', Text),
-            ('&.*?;', Name.Entity),
+            (r'&\S*?;', Name.Entity),
             (r'\<\!\[CDATA\[.*?\]\]\>', Comment.Preproc),
             ('<!--', Comment, 'comment'),
             (r'<\?.*?\?>', Comment.Preproc),
             ('<![^>]*>', Comment.Preproc),
-            (r'<\s*[a-zA-Z0-9:.-]+', Name.Tag, 'tag'),
-            (r'<\s*/\s*[a-zA-Z0-9:.-]+\s*>', Name.Tag),
+            (r'<\s*[a-zA-Z0-9:._-]+', Name.Tag, 'tag'),
+            (r'<\s*/\s*[a-zA-Z0-9:._-]+\s*>', Name.Tag),
         ],
         'comment': [
             ('[^-]+', Comment),
@@ -389,3 +533,35 @@ class XmlLexer(RegexLexer):
     def analyse_text(text):
         if looks_like_xml(text):
             return 0.5
+
+
+class XsltLexer(XmlLexer):
+    '''
+    A lexer for XSLT.
+
+    *New in Pygments 0.10.*
+    '''
+
+    name = 'XSLT'
+    aliases = ['xslt']
+    filenames = ['*.xsl', '*.xslt']
+
+    EXTRA_KEYWORDS = set([
+        'apply-imports', 'apply-templates', 'attribute',
+        'attribute-set', 'call-template', 'choose', 'comment',
+        'copy', 'copy-of', 'decimal-format', 'element', 'fallback',
+        'for-each', 'if', 'import', 'include', 'key', 'message',
+        'namespace-alias', 'number', 'otherwise', 'output', 'param',
+        'preserve-space', 'processing-instruction', 'sort',
+        'strip-space', 'stylesheet', 'template', 'text', 'transform',
+        'value-of', 'variable', 'when', 'with-param'
+    ])
+
+    def get_tokens_unprocessed(self, text):
+        for index, token, value in XmlLexer.get_tokens_unprocessed(self, text):
+            m = re.match('</?xsl:([^>]*)/?>?', value)
+
+            if token is Name.Tag and m and m.group(1) in self.EXTRA_KEYWORDS:
+                yield index, Keyword, value
+            else:
+                yield index, token, value
