@@ -5,24 +5,29 @@
 
     pygments version of my "native" vim theme.
 
-    :copyright: 2006 by Armin Ronacher.
+    :copyright: 2006-2007 by Armin Ronacher.
     :license: BSD, see LICENSE for more details.
 """
 
 from pygments.style import Style
 from pygments.token import Keyword, Name, Comment, String, Error, \
-     Number, Operator, Generic, Token
+     Number, Operator, Generic, Token, Whitespace
 
 
 class NativeStyle(Style):
+    """
+    Pygments version of the "native" vim theme.
+    """
 
     background_color = '#202020'
 
     styles = {
         Token:              '#d0d0d0',
+        Whitespace:         '#666666',
 
         Comment:            'italic #999999',
         Comment.Preproc:    'noitalic bold #cd2828',
+        Comment.Special:    'noitalic bold #e50808 bg:#520000',
 
         Keyword:            'bold #6ab825',
         Keyword.Pseudo:     'nobold',
