@@ -1,14 +1,14 @@
 # -*- coding: utf-8 -*-
 """
     Pygments tests with example files
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: 2006 by Georg Brandl.
-    :license: GNU GPL, see LICENSE for more details.
+    :copyright: 2006-2007 by Georg Brandl.
+    :license: BSD, see LICENSE for more details.
 """
 
-import unittest
 import os
+import unittest
 
 from pygments import highlight
 from pygments.lexers import get_lexer_for_filename, get_lexer_by_name
@@ -38,11 +38,12 @@ for fn in os.listdir(os.path.join(testdir, 'examplefiles')):
     def test(self, lx=lx, absfn=absfn):
         text = file(absfn, 'U').read()
         text = text.strip('\n') + '\n'
-        ntext = ''
+        text = text.decode('latin1')
+        ntext = []
         for type, val in lx.get_tokens(text):
-            ntext += val
+            ntext.append(val)
             self.failIf(type == Error, 'lexer generated error token for '+absfn)
-        if ntext != text:
+        if u''.join(ntext) != text:
             self.fail('round trip failed for '+absfn)
 
     setattr(ExampleFileTest, 'test_%i' % lfd, test)
